﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;

namespace CfChart
{
    /// <summary>
    /// 饼图
    /// </summary>
    /// <remarks>饼图仅支持一个数据系列</remarks>
    public partial class PieChart : Chart
    {
        /// <summary>
        /// 中心点
        /// </summary>
        private Point center;

        public PieChart()
        {
            this.InitializeComponent();
        }

        /// <summary>
        /// 绘制图表
        /// </summary>
        protected override void DrawChart()
        {
            this.center = new Point(Width / 2, Convert.ToInt32(Width / 2 / Math.Sqrt(2)));

            int serialCount = DataSeries[0].Serials.Count;
            double[] datas = new double[serialCount];
            double sum = 0;
            for (int i = 0; i < serialCount; i++)
            {
                datas[i] = DataSeries[0].Serials[i].Data;
                sum += datas[i];
            }

            int a = 100, b = 71;
            double startAngle = 0;

            for (int i = 0; i < serialCount; i++)
            {
                double endAngle = datas[i] / sum * 2 * Math.PI + startAngle;

                List<Point> points = new List<Point>();
                double angle = startAngle;

                while (angle < endAngle)
                {
                    points.Add(GetPoint(this.center, a, b, angle));
                    angle += 0.1;
                }

                points.Add(GetPoint(this.center, a, b, endAngle));
                points.Add(this.center);

                startAngle = endAngle;

                graphics.FillPolygon(new SolidBrush(DataSeries[0].Serials[i].Color), points.ToArray());
            }
        }

        /// <summary>
        /// 绘制图例
        /// </summary>
        protected override void DrawLegend()
        {
            List<Serial> serials = DataSeries[0].Serials;
            for (int i = 0; i < serials.Count; i += 2)
            {
                Label label = new Label();
                label.Left = 30;
                label.Top = this.center.Y * 2 + 3 + i * 10;
                label.Size = new Size(90, 20);
                label.Text = serials[i].Label;
                Controls.Add(label);
                graphics.FillRectangle(new SolidBrush(serials[i].Color), 3, label.Top + 2, 25, label.Height - 5);

                label = new Label();
                label.Left = 150;
                label.Top = this.center.Y * 2 + 3 + i * 10;
                label.Size = new Size(90, 20);
                label.Text = serials[i + 1].Label;
                Controls.Add(label);
                graphics.FillRectangle(new SolidBrush(serials[i + 1].Color), 123, label.Top + 2, 25, label.Height - 5);
            }
        }

        /// <summary>
        /// 设置颜色
        /// </summary>
        protected override void SetColorForData()
        {
            int count = DataSeries[0].Serials.Count;
            Color[] colors = new PredefinedColors(count).Colors;
            for (int i = 0; i < count; i++)
            {
                DataSeries[0].Serials[i].Color = colors[i];
            }
        }

        /// <summary>
        /// 获取弧线上的点
        /// </summary>
        /// <param name="center">中心点</param>
        /// <param name="a">椭圆长半径</param>
        /// <param name="b">椭圆短半径</param>
        /// <param name="angle">弧度</param>
        /// <returns>弧线上的点</returns>
        private static Point GetPoint(Point center, int a, int b, double angle)
        {
            Point point = new Point();
            double tan = Math.Tan(angle);
            double xD = a * b * Math.Sqrt(1 / (a * a * tan * tan + b * b));

            if (angle <= Math.PI / 2)
            {
                point.X = center.X + Convert.ToInt32(xD);
                point.Y = center.Y - Convert.ToInt32(xD * tan);
            }
            else if (angle <= Math.PI)
            {
                point.X = center.X - Convert.ToInt32(xD);
                point.Y = center.Y + Convert.ToInt32(xD * tan);
            }
            else if (angle <= Math.PI * 1.5)
            {
                point.X = center.X - Convert.ToInt32(xD);
                point.Y = center.Y + Convert.ToInt32(xD * tan);
            }
            else if (angle <= Math.PI * 2)
            {
                point.X = center.X + Convert.ToInt32(xD);
                point.Y = center.Y - Convert.ToInt32(xD * tan);
            }

            return point;
        }
    }
}